﻿
# Python imports
import os

# Tornado imports
import tornado.auth
import tornado.httpserver
import tornado.ioloop
import tornado.options
import tornado.web

from tornado.options import define, options
from tornado.web import url

# Sqlalchemy imports
from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker

# App imports
import handlers as h
import models
import binders
import uimodules
import utils

# Options
define('port', default=8888, help='run on the given port', type=int)
define('debug', default=True, type=bool)

# SQLite
#define('db_path', default='sqlite:///:memory:', type=str)
#define('db_path', default='sqlite:////tmp/test.db', type=str)

# SQL Server -> DOWNLOAD FROM: http://code.google.com/p/pyodbc/
#            -> Documentation at: http://docs.sqlalchemy.org/en/latest/dialects/mssql.html#id1
#define('db_path', default='mssql+pyodbc://admin:admin@localhost/testdb', type=str)

define('db_path', default='mssql+pyodbc://admin:admin@localhost/Northwind', type=str)

class Application(tornado.web.Application):
  def __init__(self):
    handlers = [
      #application handlers here
      url(r'/', h.HomeHandler, name='home'),
      #db editor handlers here
      url(r'/db/(\w+)', h.DbHandler),
      url(r'/db/', h.IndexHandler, name='index'),
    ]
    settings = dict(
      debug=options.debug,
      static_path=os.path.join(os.path.dirname(__file__), 'static'),
      template_path=os.path.join(os.path.dirname(__file__), 'templates'),
      xsrf_cookies=True,
      cookie_secret='bZJc2sWbQLKos6GkHn/VB9oXwQt8S0R0kRvJ5/xJ89E=',
      #cookie_secret=utils.id_generator(55),
      ui_modules=uimodules,
    )
    tornado.web.Application.__init__(self, handlers, **settings)
    engine = create_engine(options.db_path, convert_unicode=True, echo=options.debug)
    models.init_db(engine)
    self.db = scoped_session(sessionmaker(bind=engine))
    self.modelbinders = binders.ModelBinders()

def main():
  tornado.options.parse_command_line()
  http_server = tornado.httpserver.HTTPServer(Application())
  http_server.listen(options.port)
  tornado.ioloop.IOLoop.instance().start()

if __name__ == '__main__':
  main()

