﻿# Python imports
import os
import sys

# Tornado imports
import tornado.auth
import tornado.httpserver
import tornado.ioloop
import tornado.options
import tornado.web

from tornado.options import define, options
from tornado.web import url

# Sqlalchemy imports
from sqlalchemy import create_engine, or_, and_, desc
from sqlalchemy.orm import scoped_session, sessionmaker

# App imports
import textwrap
import models
import uimodules
import utils

class BaseHandler(tornado.web.RequestHandler):
  @property
  def db(self):
    return self.application.db

  def write_error(self, status_code, **kwargs):
    import traceback
    if self.settings.get("debug") and "exc_info" in kwargs:
      exc_info = kwargs["exc_info"]
      trace_info = ''.join(["%s<br/>" % line for line in traceback.format_exception(*exc_info)])
      request_info = ''.join(["<tr><td>%s</td><td>%s</td></tr>" % (k, self.request.__dict__[k] ) for k in self.request.__dict__.keys()])
      error = exc_info[1]
      self.render('error.html', status_code=status_code, error=error, trace_info=trace_info, request_info=request_info)
    else:
      self.render('error.html', status_code=status_code, error=None, trace_info=None, request_info=None)

class HomeHandler(BaseHandler):
  def get(self):
    self.redirect("/db/")

class IndexHandler(BaseHandler):
  def get(self):
    self.render('index.html')
    
class DbHandler(BaseHandler):  
  def get_binder(self, input):
    return self.application.modelbinders.get_binder(input)

  def get(self, input):
    self.error = None
    self.input = input
    self.binder = self.get_binder(input)
    action = self.get_argument('action', None)
    if(action == 'edit'):
      self.get_edit()
    elif(action == 'add'):
      self.get_add()
    elif(action == 'view'):
      self.get_view()
    elif(action == 'search'):
      self.get_search()
    elif(action == 'delete'):
      self.delete_existing()
    else:
      self.get_list()
  
  def post(self, input):
    self.error = None
    self.input = input
    self.binder = self.get_binder(input)
    action = self.get_argument('action', None)
    if(action == 'edit'):
      self.post_update()
    elif(action == 'add'):
      self.post_new()
    elif(action == 'delete'):
      self.delete_existing()
      
  def get_list(self):
    instances, pager = self.binder.fetch_using_filter(self)
    self.render('entity_list.html', input=self.input, data=instances, pager=pager)

  def get_edit(self):
    instance, related = self.binder.fetch_using_pk(self)
    self.render('entity_edit.html', input=self.input, data=instance, related=related, error=self.error)

  def get_view(self):
    instance, related = self.binder.fetch_using_pk(self)
    self.render('entity_view.html', input=self.input, data=instance, related=related)

  def get_add(self):
    model = self.binder.get_model(self)
    related = self.binder.fetch_related(self)
    self.render('entity_add.html', input=self.input, data=model, related=related, error=self.error)
  
  def get_search(self):
    model = self.binder.get_model(self)
    related = self.binder.fetch_related(self)
    self.render('entity_search.html', input=self.input, data=model, related=related)
    
  def post_new(self):
    try:
      instance = self.binder.create(self)
      self.redirect("/db/"+self.input)
    except Exception as e:
      self.error = e
      self.get_add();
    
  def post_update(self):
    try:
      instance = self.binder.update(self)
      self.redirect("/db/"+self.input)
    except Exception as e:
      self.error = e
      self.get_edit();
    
  def delete_existing(self):
    self.binder.delete(self)
    self.redirect("/db/"+self.input)  
